Package: calculus
Type: Package
Title: High Dimensional Numerical and Symbolic Calculus
Version: 1.1.0
Authors@R: person(given = "Emanuele", family = "Guidotti", email = "emanuele.guidotti@usi.ch", role = c("aut", "cre"), comment = c(ORCID = "0000-0002-8961-6623"))
Description: Efficient C++ optimized functions for numerical and symbolic calculus as described in Guidotti (2022) <doi:10.18637/jss.v104.i05>. It includes basic arithmetic, tensor calculus, Einstein summing convention, fast computation of the Levi-Civita symbol and generalized Kronecker delta, Taylor series expansion, multivariate Hermite polynomials, high-order derivatives, ordinary differential equations, differential operators (Gradient, Jacobian, Hessian, Divergence, Curl, Laplacian) and numerical integration in arbitrary orthogonal coordinate systems: cartesian, polar, spherical, cylindrical, parabolic or user defined by custom scale factors. 
License: GPL-3
URL: https://calculus.eguidotti.com
BugReports: https://github.com/eguidotti/calculus/issues
LinkingTo: Rcpp
Imports: Rcpp (>= 1.0.1)
Suggests: cubature, testthat, knitr, rmarkdown
RoxygenNote: 7.2.3
VignetteBuilder: knitr
NeedsCompilation: yes
Packaged: 2025-06-17 16:56:35 UTC; eguidotti
Author: Emanuele Guidotti [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-8961-6623>)
Maintainer: Emanuele Guidotti <emanuele.guidotti@usi.ch>
Repository: CRAN
Date/Publication: 2025-06-18 08:20:02 UTC
