# Generated by roxygen2: do not edit by hand

export("%cross%")
export("%curl%")
export("%diff%")
export("%div%")
export("%divergence%")
export("%dot%")
export("%gradient%")
export("%hessian%")
export("%inner%")
export("%jacobian%")
export("%kronecker%")
export("%laplacian%")
export("%mx%")
export("%outer%")
export("%prod%")
export("%sum%")
export("diagonal<-")
export("index<-")
export(c2e)
export(contraction)
export(cross)
export(curl)
export(delta)
export(derivative)
export(diagonal)
export(divergence)
export(e2c)
export(einstein)
export(epsilon)
export(evaluate)
export(gradient)
export(hermite)
export(hessian)
export(index)
export(integral)
export(jacobian)
export(laplacian)
export(mx)
export(mxdet)
export(mxinv)
export(mxtr)
export(ode)
export(partitions)
export(taylor)
export(wrap)
import(Rcpp)
importFrom(Rcpp,evalCpp)
useDynLib(calculus, .registration=TRUE)
