# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

cpp_partitions <- function(n, max = 0L, length = 0L, perm = FALSE, fill = FALSE, equal = TRUE) {
    .Call(`_calculus_cpp_partitions`, n, max, length, perm, fill, equal)
}

cpp_parity <- function(x, y) {
    .Call(`_calculus_cpp_parity`, x, y)
}

cpp_paste <- function(x, y, sep) {
    .Call(`_calculus_cpp_paste`, x, y, sep)
}

cpp_collapse <- function(x, sep) {
    .Call(`_calculus_cpp_collapse`, x, sep)
}

cpp_inner <- function(x, y) {
    .Call(`_calculus_cpp_inner`, x, y)
}

cpp_outer <- function(x, y) {
    .Call(`_calculus_cpp_outer`, x, y)
}

cpp_einstein <- function(x, y, dim, drop = TRUE) {
    .Call(`_calculus_cpp_einstein`, x, y, dim, drop)
}

cpp_trace <- function(x, dim, drop = TRUE) {
    .Call(`_calculus_cpp_trace`, x, dim, drop)
}

cpp_det <- function(x, n) {
    .Call(`_calculus_cpp_det`, x, n)
}

